/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2013 Hajime Tazaki, NICT
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Hajime Tazaki <tazaki@nict.go.jp>
 */

#include "ns3/object-factory.h"
#include "ns3/names.h"
#include "ns3/log.h"
#include "dce-submod-helper.h"
#include <sys/stat.h>
#include <fstream>

NS_LOG_COMPONENT_DEFINE ("DceSubModuleHelper");

namespace ns3 {

class DceSubModuleConfig : public Object
{
private:
  static int index;
  std::string router_id;
  std::map<std::string, uint32_t> *networks;
public:
  DceSubModuleConfig ()
  {
  }
  ~DceSubModuleConfig ()
  {
  }

  static TypeId
  GetTypeId (void)
  {
    static TypeId tid = TypeId ("ns3::DceSubModuleConfig")
      .SetParent<Object> ()
      .AddConstructor<DceSubModuleConfig> ()
    ;
    return tid;
  }
  TypeId
  GetInstanceTypeId (void) const
  {
    return GetTypeId ();
  }

  void
  SetFilename (const std::string &filename)
  {
    m_filename = filename;
  }

  std::string
  GetFilename () const
  {
    return m_filename;
  }

  std::string m_filename;

  virtual void
  Print (std::ostream& os) const
  {
    os << "hostname localhost" << std::endl
       << "password deadbeaf" << std::endl
       << "log stdout" << std::endl;
  }
};
std::ostream& operator << (std::ostream& os, DceSubModuleConfig const& config)
{
  config.Print (os);
  return os;
}


DceSubModuleHelper::DceSubModuleHelper ()
{
}

void
DceSubModuleHelper::SetAttribute (std::string name, const AttributeValue &value)
{
}


void
DceSubModuleHelper::GenerateConfig (Ptr<Node> node)
{
  Ptr<DceSubModuleConfig> conf = node->GetObject<DceSubModuleConfig> ();

  // config generation
  std::stringstream conf_dir, conf_file;
  // FIXME XXX
  conf_dir << "files-" << node->GetId () << "";
  ::mkdir (conf_dir.str ().c_str (), S_IRWXU | S_IRWXG);
  conf_dir << "/usr/";
  ::mkdir (conf_dir.str ().c_str (), S_IRWXU | S_IRWXG);
  conf_dir << "/local/";
  ::mkdir (conf_dir.str ().c_str (), S_IRWXU | S_IRWXG);
  conf_dir << "/etc/";
  ::mkdir (conf_dir.str ().c_str (), S_IRWXU | S_IRWXG);

  conf_file << conf_dir.str () << "/sample_app.conf";
  conf->SetFilename ("/usr/local/etc/sample_app.conf");

  std::ofstream conff;
  conff.open (conf_file.str ().c_str ());
  conff << *conf;

  conff << "additional config 1: True" << std::endl;

  conff.close ();
}

ApplicationContainer
DceSubModuleHelper::Install (Ptr<Node> node)
{
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
DceSubModuleHelper::Install (std::string nodeName)
{
  Ptr<Node> node = Names::Find<Node> (nodeName);
  return ApplicationContainer (InstallPriv (node));
}

ApplicationContainer
DceSubModuleHelper::Install (NodeContainer c)
{
  ApplicationContainer apps;
  for (NodeContainer::Iterator i = c.Begin (); i != c.End (); ++i)
    {
      apps.Add (InstallPriv (*i));
    }

  return apps;
}

ApplicationContainer
DceSubModuleHelper::InstallPriv (Ptr<Node> node)
{
  DceApplicationHelper process;
  ApplicationContainer apps;

  Ptr<DceSubModuleConfig> conf = node->GetObject<DceSubModuleConfig> ();
  if (!conf)
    {
      conf = new DceSubModuleConfig ();
      node->AggregateObject (conf);
    }
  GenerateConfig (node);
  process.SetBinary ("sample_app");
  process.AddArguments ("-f", conf->GetFilename ());
  process.SetStackSize (1 << 16);
  apps.Add (process.Install (node));
  apps.Get (0)->SetStartTime (Seconds (1.0 + 0.01 * node->GetId ()));
  node->AddApplication (apps.Get (0));

  return apps;
}

} // namespace ns3
